@@ -1,6 +1,6 @@
 #' One step to get the reporter score of your KO abundance table.
 #'
-#' @param kodf KO_abundance table, rowname is ko id (e.g. K00001),colnames is samples.
+#' @param kodf KO_abundance table, rowname are feature ids (e.g. K00001 if feature="ko"; PEX11A if feature="gene"; C00024 if feature="compound"), colnames are samples.
 #' @param group The comparison groups (at least two categories) in your data, one column name of metadata when metadata exist or a vector whose length equal to columns number of kodf. And you can use factor levels to change order.
 #' @param metadata sample information data.frame contains group
 #' @param mode 'mixed' or 'directed' (default, only for two groups differential analysis or multi-groups correlation analysis.), see details in \code{\link{pvalue2zs}}.
@@ -113,7 +113,7 @@ reporter_score <- function(
   }
   ko_pvalue <- ko.test(kodf, group, metadata, method = method, pattern = pattern, threads = threads, p.adjust.method1 = p.adjust.method1, verbose = verbose)
   if (verbose) {
-    pcutils::dabiao("2.Transfer p.value to z-score")
+    pcutils::dabiao("2.Transfer p.value to Z-score")
   }
   ko_stat <- pvalue2zs(ko_pvalue, mode = mode, p.adjust.method1 = p.adjust.method1)
   attributes(ko_stat)$check <- TRUE
@@ -137,6 +137,41 @@ reporter_score <- function(
   res
 }
 
+#' Combine the results of 'step by step GRSA'
+#'
+#' @inheritParams reporter_score
+#' @param ko_stat result of \code{\link{pvalue2zs}}
+#' @param reporter_s result of \code{\link{get_reporter_score}}
+#'
+#' @return reporter_score object
+#' @export
+#' @family GRSA
+#' @examples
+#' \donttest{
+#' data("KO_abundance_test")
+#' ko_pvalue <- ko.test(KO_abundance, "Group", metadata)
+#' ko_stat <- pvalue2zs(ko_pvalue, mode = "directed")
+#' reporter_s1 <- get_reporter_score(ko_stat, perm = 499)
+#' reporter_res <- combine_rs_res(KO_abundance, "Group", metadata, ko_stat, reporter_s1)
+#' }
+combine_rs_res <- function(kodf, group, metadata, ko_stat, reporter_s, modulelist = NULL) {
+  if (is.null(modulelist)) {
+    if (!is.null(attributes(reporter_s)$type)) {
+      modulelist <- get_modulelist(
+        type = attributes(reporter_s)$type, feature = attributes(reporter_s)$feature,
+        verbose = FALSE, chr = TRUE
+      )
+    } else {
+      stop("modulelist is NULL as you may customize it, please set modulelist.")
+    }
+  }
+  res <- list(
+    kodf = kodf, ko_stat = ko_stat, reporter_s = reporter_s,
+    modulelist = modulelist, group = group, metadata = metadata
+  )
+  class(res) <- "reporter_score"
+  res
+}
 
 #' Differential analysis or Correlation analysis for KO-abundance table
 #'
@@ -155,7 +190,7 @@ ko.test <- function(kodf, group, metadata = NULL, method = "wilcox.test", patter
   i <- NULL
   t1 <- Sys.time()
 
-  method <- match.arg(method, c("t.test", "wilcox.test", "kruskal.test", "anova", "pearson", "kendall", "spearman", "lm"))
+  method <- match.arg(method, c("t.test", "wilcox.test", "kruskal.test", "anova", "pearson", "kendall", "spearman", "lm", "none"))
   if (!is.null(pattern)) {
     if (!method %in% c("pearson", "kendall", "spearman")) {
       stop("method should be one of \"pearson\", \"kendall\", \"spearman\" when you specify a pattern")
@@ -271,6 +306,9 @@ ko.test <- function(kodf, group, metadata = NULL, method = "wilcox.test", patter
   # main function
   loop <- function(i) {
     val <- tkodf[, i]
+    if (method == "none") {
+      return(NA)
+    }
     if (method == "wilcox.test") {
       pval <- stats::wilcox.test(val ~ group)$p.value
     }
@@ -367,7 +405,7 @@ ko.test <- function(kodf, group, metadata = NULL, method = "wilcox.test", patter
 
 #' Transfer p-value of KOs to Z-score
 #'
-#' @param ko_pvalue data.frame from \code{\link{ko.test}}
+#' @param ko_pvalue data.frame from \code{\link{ko.test}}, `KO_id` and `p.value` columns are required.
 #' @inheritParams reporter_score
 #'
 #' @return ko_stat data.frame
@@ -453,8 +491,12 @@ pvalue2zs <- function(ko_pvalue, mode = c("directed", "mixed")[1], p.adjust.meth
     message("detect the origin_p.value, use the origin_p.value")
     res.dt$p.value <- res.dt$origin_p.value
   }
-  if (!all(c("p.value") %in% colnames(res.dt))) {
-    stop("check if `p.value` in your ko_stat dataframe!")
+  if (!all(c("KO_id", "p.value") %in% colnames(res.dt))) {
+    stop("check if `KO_id` and `p.value` in colnames of your ko_pvalue dataframe!")
+  }
+
+  if (all(is.na(res.dt$p.value))) {
+    stop("all `p.value` are NA!, please check your ko_pvalue dataframe!")
   }
 
   stopifnot(mode %in% c("mixed", "directed"))
@@ -497,7 +539,8 @@ pvalue2zs <- function(ko_pvalue, mode = c("directed", "mixed")[1], p.adjust.meth
     attributes(res.dt)$mode <- "mixed"
   }
   if (mode == "directed") {
-    if (!"type" %in% colnames(res.dt)) {
+    if (!"sign" %in% colnames(res.dt)) {
+      message("No `sign` in colnames of your ko_pvalue dataframe, use the `diff_mean` column to determine the sign of ko!")
       stop("directed mode only use for two groups differential analysis or multi-groups correlation analysis.")
     }
     res.dt$origin_p.value <- ko_pvalue$p.value
@@ -523,7 +566,7 @@ pvalue2zs <- function(ko_pvalue, mode = c("directed", "mixed")[1], p.adjust.meth
     zs <- ifelse(zs > 8.209536, 8.209536, zs)
     zs <- ifelse(zs < -8.209536, -8.209536, zs)
 
-    # 通过判断上下调给予z-score正负号，让最后的reporter-score正负号作为上下调标志
+    # 通过判断上下调给予Z-score正负号，让最后的reporter-score正负号作为上下调标志
     res.dt$Z_score <- ifelse(res.dt$sign < 0, -zs, zs)
     attributes(res.dt)$mode <- "directed"
   }
@@ -639,7 +682,7 @@ get_reporter_score <- function(
   check_kodf_modulelist(ko_stat, type, feature, modulelist, verbose, mode = 2)
 
   if (is.null(modulelist)) {
-    modulelist <- get_modulelist(type, feature, verbose)
+    modulelist <- get_modulelist(type, feature, verbose = verbose)
     type_flag <- TRUE
   }
 
@@ -1079,6 +1122,7 @@ up_level_KO <- function(KO_abundance, level = "pathway", show_name = FALSE, modu
 
   b <- pcutils::hebing(dplyr::select(aa, -c("KO_id", "Pathway")), aa$Pathway, 1, act = "sum")
   if (show_name) {
+    path2name <- setNames(make.unique(path2name), names(path2name))
     rownames(b) <- c(path2name, Unknown = "Unknown")[rownames(b)]
   }
   b
